local AceGUI = LibStub("AceGUI-3.0")


--------------------------
-- Check Box			--
--------------------------
--[[
	Events :
		OnValueChanged

]]
do
	local Type = "UIF-CheckBox"
	local Version = 3
	
	local function SetFont(self, font, size, flags)
		self.text:SetFont(font, size, flags)
	end
	
	local function SetFontColor(self, color)
		self.newColor = color
		self.text:SetTextColor(unpack(color))
	end
	
	local function SetDisabled(self, disabled)
		self.oldSetDisable(self, disabled)
		if not disabled and self.newColor then
			self.text:SetTextColor(unpack(self.newColor))
		end
	end
	
	local function SetValue(self, value)
		self.oldSetValue(self, value)
		SetDisabled(self, self.disabled)
	end
	
	local function CheckBox_OnMouseDown(frame)
		local self = frame.obj
		if not self.disabled then
			local img = self.image:GetTexture()
			if img then
				self.text:SetPoint("LEFT", self.image, "RIGHT", 1, -1)
			else
				self.text:SetPoint("LEFT", self.checkbg, "RIGHT", 1, -1)
			end
		end
		AceGUI:ClearFocus()
	end

	local function CheckBox_OnMouseUp(frame)
		local self = frame.obj
		if not self.disabled then
			self:ToggleChecked()

			if self.checked then
				PlaySound("igMainMenuOptionCheckBoxOn")
			else -- for both nil and false (tristate)
				PlaySound("igMainMenuOptionCheckBoxOff")
			end

			self:Fire("OnValueChanged", self.checked)
			
			local img = self.image:GetTexture()
			if img then
				self.text:SetPoint("LEFT", self.image,"RIGHT", 1, 0)
			else
				self.text:SetPoint("LEFT", self.checkbg, "RIGHT")
			end
		end
	end
	
	local function Constructor()
		local self = AceGUI:Create("CheckBox")
		self.type = Type
		
		self.oldSetDisable = self.SetDisabled
		self.oldSetValue = self.SetValue
		self.SetDisabled = SetDisabled
		self.SetValue = SetValue
		
		self.frame:SetScript("OnMouseDown", CheckBox_OnMouseDown)
		self.frame:SetScript("OnMouseUp", CheckBox_OnMouseUp)

		
		self.SetFont = SetFont
		self.SetFontColor = SetFontColor
		return self
	end
	
	AceGUI:RegisterWidgetType(Type,Constructor,Version)
end
