local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
if not RW then return end
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Base")
local MapData = LibStub("LibMapData-1.0")


local Utils = {}
RW.Utils = Utils

local Party
local math_modf = math.modf

function Utils:InTable(t, entry)
	for i, v in pairs(t) do
		if v == entry then return true end
	end
end

function Utils:InTable2(t, key, entry)
	for i, v in pairs(t) do
		if v[key] == entry then return true end
	end
end

function Utils:GetTableData(t, key, entry)
	for i, v in pairs(t) do
		if v[key] == entry then return v end
	end
end

function Utils:GetIndexForvalue(t, value)
	for i, v in pairs(t) do
		if v == value then return i end
	end
end

function Utils:NrElements(t)
	local count = 0
	for i, v in pairs(t) do
		count = count + 1
	end
	return count
end

function Utils:TRemove(t, entry)
	for i, v in pairs(t) do
		if v == entry then 
			tremove(t, i)
		end
	end
end

function Utils:CopyTable(t, d)
	if (d  or 0) > 5 then return nil end
	local new = {}
	for i, v in pairs(t) do
		if type(v) == "table" then
			new[i] = self:CopyTable(v, (d or 0) + 1)
		else
			new[i] = v
		end
	end
	return new
end

function Utils:CopyTableShallow(s, d)
	if type(s) ~= "table" or type(d) ~= "table" then return end
	
	for i, v in pairs(s) do
		if type(v) == "table" then
			self:CopyTableShallow(v, d[i])
		else
			d[i] = v
		end
	end
end

function Utils:TimeToStr(time)
	if type(time) ~= "number" then return time end
	time = math.floor(time)
	local t
	if time < 60 then
		return L.COMBATTIME_SEC:format(time)
	elseif time % 60 == 0 then
		return L.COMBATTIME_MIN:format(time/60)
	else
		return L.COMBATTIME:format(time/60, time % 60)
	end
end

function Utils:TimeToStringShort(t)
	if type(t) ~= "number" then return t end
	if t <= 10 then
		return ("%.1fs"):format(t)
	elseif t <= 60 then
		return ("%ds"):format(t)
	else
		return ("%0.2d:%0.2d"):format(t/60, math.fmod(t, 60))
	end
end

function Utils:RGBConvert(r, g, b, a)
	return r/255, g/255, b/255, (a and a/255 or 1)
end

function Utils:Lerp(s, e, a)
	if not s or not e or not a then return 0 end
	return s + ((e - s) * a)
end

function Utils:ColorNames(str)
	if not Party then Party = RW:GetPlugin("Party") end
	str = str:gsub(">([%S\s-]+)<", function(name)
		if Party.group[name] then
			local color = RAID_CLASS_COLORS[Party.group[name].class]
			if color then
				name = ("|cff%.2x%.2x%.2x%s|r"):format(color.r * 255, color.g * 255, color.b * 255, name)
			end
		end
		return name
	end)
	return str
end

-- A bit of hack to solve if the first argument is correct but the rest is not, 
-- only for timer and message formating realy.
function Utils:Format(text, ...)
	local ok, str = pcall(format, text, ...)
	if not ok then
		local first = select(1, ...)
		ok, str = pcall(format, text, first, L.UNKNOWN, 0)
	end
	if not ok then
		local first = select(1, ...)
		ok, str = pcall(format, text, first, 0)
	end
	return ok and str or text:gsub("(%%.)", L.UNKNOWN)
end

Utils.SpellName = setmetatable({}, {
	__index = function(t, key)
		if type(key) ~= "number" then return key end
		local name = GetSpellInfo(key)
		if not name then 
			print(("Cannot fetch spell name for %q"):format(key)) 
			return key
		end
		return name		
	end}
)

do
	local playerSpells
	function Utils:GetRandomSpell()
		if not playerSpells then
			playerSpells = {}
			local i = 1
			while true do
				local spellName = GetSpellName(i, BOOKTYPE_SPELL)
				if not spellName then break end
				playerSpells[#playerSpells + 1] = spellName
				i = i + 1
			end
		end
		local name, rank, icon = GetSpellInfo(playerSpells[math.random(1, #playerSpells)])
		return name, rank, icon
	end
end

-- Tekkub's ColorGradient() http://www.wowwiki.com/ColorGradient
function Utils:ColorGradient(perc, r1, g1, b1, r2, g2, b2, r3, g3, b3)
	if perc >= 1 then return r3, g3, b3 elseif perc <= 0 then return r1, g1, b1 end
	local segment, relperc = math_modf(perc*2)
	if segment == 1 then r1, g1, b1, r2, g2, b2 = r2, g2, b2, r3, g3, b3 end
	return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
end

function Utils:ColorGradientSimple(perc, r1, g1, b1, r2, g2, b2)
	if perc >= 1 then return r2, g2, b2
	elseif perc <= 0 then return r1, g1, b1
	end
	return r1 + (r2-r1)*perc, g1 + (g2 - g1)*perc, b1 + (b2 - b1)*perc
end

-----------------------------
-- Party functions
function Utils:IsHeroic()
	local name, instanceType, difficultyIndex, difficultyName, maxPlayers, playerDifficulty, isDynamic = GetInstanceInfo()
	if (instanceType == "party" and difficultyIndex == 2) or difficultyIndex > 2 then return true end
	if instanceType == "none" then return false end
end

function Utils:InstanceSize()
	local _, instanceType, difficulty, _, maxPlayers = GetInstanceInfo()
	if instanceType == "none" then return 10 end
	return maxPlayers
end

-- Constants
Utils.Constants = {}
Utils.Constants.FontFlagList = {
	["NONE"] = "None",
	["OUTLINE"] = "Outline",
	["THICKOUTLINE"] = "Thick Outline",
	["MONOCHROME"] = "Monochrome",
}

Utils.Constants.RaidIconNameToNumber = {
	STAR	= 1,
	CIRCLE 	= 2,
	DIAMOND	= 3,
	TRIANGLE = 4,
	MOON 	= 5,
	SQUARE 	= 6,
	CROSS 	= 7,
	SKULL 	= 8,
}

-----------------------------
-- Frames
function Utils:CreateAnchor(name, parent, text)
	local frame = CreateFrame("Frame", name or nil, parent or nil)
	frame:SetPoint("CENTER")
	frame:SetWidth(200)
	frame:SetHeight(25)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetFrameStrata("BACKGROUND")
	frame:SetMovable(true)

	local bg = frame:CreateTexture(nil, "PARENT")
	bg:SetAllPoints(frame)
	bg:SetTexture(1, 1, 1, 1)
	bg:SetGradient("HORIZONTAL", 0.39, 0.55, 0.87, 0.1, 0.1, 0.1)
	frame.background = bg
	
	local str = frame:CreateFontString(name and name.."Text", "ARTWORK")
	str:SetFont(LibStub("LibSharedMedia-3.0"):Fetch("font", "accid"), 16, "OUTLINE")
	str:SetAllPoints()
	str:SetText(text)
	
	--hide button
	local closebutton = CreateFrame("Button", nil, frame)
	closebutton:SetWidth(20)
	closebutton:SetHeight(20)
	closebutton:SetPoint("RIGHT", -4, 0)
	
	local t = closebutton:CreateTexture(nil, "BACkGROUND")
	t:SetAllPoints(closebutton)
	t:SetTexture([[Interface\AddOns\RaidWatch\Media\Icons\close.tga]])
	t:SetVertexColor(0.5, 0.5, 0.5)
	closebutton.texture = t
	
	closebutton.frame = frame
	closebutton:EnableMouse(true)
	closebutton:SetScript("OnClick", function(self)
		self.frame:Hide()
	end)
	closebutton:SetScript("OnEnter", function(self)
		self.texture:SetVertexColor(0.9, 0.5, 0.5)
		GameTooltip:SetOwner(self)
		GameTooltip:AddLine("Hide")
		GameTooltip:Show()
	end)
	closebutton:SetScript("OnLeave", function(self)
		self.texture:SetVertexColor(0.5, 0.5, 0.5)
		GameTooltip:Hide()
	end)
		
	frame.hidebutton = closebutton
	frame.str = str
	frame:Hide()
	return frame
end

local barCounter = 0
function Utils:CreateBar(rightClickFunc)
	local frame = CreateFrame("Frame", "RW_Bar"..barCounter, UIParent)
	frame.ID = barCounter
	barCounter = barCounter + 1
	local bar = CreateFrame("StatusBar", frame:GetName().."Bar", frame)
	local timeText = bar:CreateFontString(frame:GetName().."TimeText", "ARTWORK")
	local text = bar:CreateFontString(frame:GetName().."Text", "ARTOWRK")
	local icon = frame:CreateTexture(frame:GetName().."Icon", "ARTWORK")
	
	frame:SetPoint("CENTER")
	frame:SetWidth(255)
	bar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -3, 3)
	bar:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -3, -3)
	icon:SetPoint("TOPLEFT", frame, "TOPLEFT", 3, -3)
	icon:SetPoint("BOTTOMRIGHT", bar, "BOTTOMLEFT", -2, 0)
	timeText:SetPoint("RIGHT", bar, "RIGHT", -4, 0)
	text:SetPoint("LEFT", bar, "LEFT", 2, 0)
	text:SetPoint("RIGHT", timeText, "LEFT")
	
	bar:SetMinMaxValues(0,1)
	
	timeText:SetJustifyH("LEFT")
	timeText:SetWidth(45)
	
	frame.timeText = timeText
	frame.text = text
	frame.bar = bar
	frame.icon = icon
	
	frame.rightClickfuc = rightClickFunc
	
	frame:SetScript("OnEvent", function(self, event, arg1, arg2)
		if arg1 == "LSHIFT" or arg1 == "RSHIFT" then
			if arg2 == 1 then
				frame:EnableMouse(true)
				self:SetScript("OnMouseDown", self.rightClickfuc)
			else
				frame:EnableMouse(false)
				self:SetScript("OnMouseDown", nil)
			end
		end
	end)
	frame:Hide()
	return frame
end

do
	local function ShowTooltip(self)
		GameTooltip:SetOwner(self, "NONE")
		if self.TT_Title then GameTooltip:AddLine(self.TT_Title, nil, nil, nil, true) end
		if self.TT_Text then GameTooltip:AddLine(self.TT_Text, 0.5, 0.5, 1, true) end
		GameTooltip:Show()
	end
	
	local function HideTooltip(self)
		GameTooltip:Hide()
	end
	
	function Utils:AddTooltip(frame, text, title)
		--[===[@debug@
		assert(frame.GetObjectType, "The object to add tooltip too have to have a frame")
		--@end-debug@]===]
		
		frame.TT_Title = title
		frame.TT_Text = text
		frame:HookScript("OnEnter", ShowTooltip)
		frame:HookScript("OnLeave", HideTooltip)
	end
	
	local prot = {}
	local backdrop = {}
	
	function prot:SetBorder(border, width, color)
		backdrop.edgeFile = border
		backdrop.edgeSize = width
		self:SetBackdrop(backdrop)
		self:SetBackdropBorderColor(unpack(color))
	end
	
	function prot:SetInsets(inset)
		self.title:SetPoint("TOPLEFT", inset, -inset)
		self.title:SetPoint("TOPRIGHT", -inset, -inset)
		self.content:SetPoint("BOTTOMRIGHT", -inset, inset)
	end
	
	function prot:SetContentHeight(height)
		self:SetHeight(height + self.title:GetHeight())
		self.contentheight = height
	end
	
	function prot:SetTitleHeight(height)
		self.title:SetHeight(height)
		self:SetContentHeight(self.contentheight)
	end
	
	function prot:SetTitle(str)
		self.title.string:SetText(str)
	end
	
	function prot:SetTitleFont(font, size, flags, color)
		self.title.string:SetFont(font, size, flags)
		self.title.string:SetTextColor(unpack(color))
	end
	
	function prot:SetTitleColor(...)
		r, g, b = ...
		self.title.background:SetGradient("HORIZONTAL", r, g, b, 0.2,0.2,0.2)
	end
	
	function prot:SetBackgroundColor(...)
		self.content.background:SetTexture(...)
	end
	
	function prot:AddTitleButton(texture, iconsize)
		self.buttons = self.buttons or {}
		local button = CreateFrame("Button", self.title:GetName().."_Button_"..#self.buttons, self.title)
		button:SetHeight(iconsize or 15)
		button:SetWidth(iconsize or 15)
		button:SetPoint("RIGHT", self.lastbutton or self.title, self.lastbutton and "LEFT" or "RIGHT", -4, 0)
		button:SetScript("OnEnter", function(self) 
			self.texture:SetVertexColor(0.9, 0.5, 0.5) 
		end)
		button:SetScript("OnLeave", function(self) 
			self.texture:SetVertexColor(0.5, 0.5, 0.5) 
		end)		
		
		local t = button:CreateTexture(nil, "ARTWORK")
		t:SetAllPoints()
		t:SetTexture(texture)
		t:SetVertexColor(0.5, 0.5, 0.5)
		button.texture = t
		
		
		tinsert(self.buttons, button)
		
		self.lastbutton = button
		
		self.title.string:SetPoint("RIGHT", button, "LEFT")
		
		return button
	end
	
	function Utils:CreateWindow(windowID)
		local frame = CreateFrame("Frame", "RW_Window_"..windowID, UIParent)
		
		local title = CreateFrame("Frame", "RW_Window_Title"..windowID, frame)
		title:SetPoint("TOPLEFT")
		title:SetPoint("TOPRIGHT")
		title:SetHeight(40)
		
		local titlestring = title:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		titlestring:SetPoint("LEFT", 2, 0)
		titlestring:SetText(windowID)
		titlestring:SetShadowOffset(1, -1)
		titlestring:SetJustifyH("LEFT")
		title.string = titlestring
		
		local background = title:CreateTexture(nil, "ARTOWRK")
		background:SetAllPoints()
		background:SetTexture(1, 1, 1, 1)
		background:SetGradient("HORIZONTAL", 1,0,1, 0.2,0.2,0.2)
		title.background = background
					
		
		local content = CreateFrame("Frame", "RW_Window_Content"..windowID, frame)
		content:SetPoint("TOPLEFT", title)
		content:SetPoint("BOTTOMRIGHT", frame)
		
		background = content:CreateTexture(nil, "ARTWORK")
		background:SetAllPoints()
		background:SetTexture(0, 0, 0, 0.5)
		content.background = background
		
		frame.title = title
		frame.content = content
		
		for i, func in pairs(prot) do
			frame[i] = func
		end
		
		return frame
	end
end

------------------------
-- Map distance stuff


do
	local CurrentZone, CurrentFloor, CurrentW, CurrentH
	MapData:RegisterCallback("MapChanged", function(event, map, floor, w, h)
		CurrentZone = map
		CurrentFloor = floor
		CurrentW = w
		CurrentH = h
	end)
	
	function Utils:GetDistanceAndAngle(x, y)
		return MapData:DistanceAndDirection(CurrentZone, CurrentFloor, x, y)
	end
	
	function Utils:GetPointInYards(x, y)
		return MapData:PointToYards (CurrentZone, CurrentFloor, x, y)
	end
	
	function Utils:GetDistanceAndAngleToUnit(uID)
		if not UnitIsConnected(uID) then return end
		local tx, ty = GetPlayerMapPosition(uID)
		return self:GetDistanceAndAngle(tx, ty)
	end
end
