----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Bars2")
if not plugin then return end

----------------------------------
-- Locals
local LSM = LibStub("LibSharedMedia-3.0")
local AGU = LibStub("AceGUI-3.0")
local UIF

local Utils = RW.Utils
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Bars")
local db

local UpdateFrame = CreateFrame("Frame")
local dragStart, dragStop


local barProt = {}
local anchorProt = {}

local AllBars, ActiveBars, Cache = {}, {}, {}
local Anchors = {}
local AnchorList = {}
local DefaultAnchor

local InConfigMode

local backdrop = {
}

function plugin:OnRegister()
	local defaults = {
		profile = {
			Anchors = {
				["**"] = {
					Position = {-0, 0},
					-- Backdrop
					Insets = 0,
					BackgroundColor = {0, 0, 0, 0.5},
					BorderColor = {1, 1, 1, 1},
					BorderWidth = 12,
					BorderTexture = "None",	
					-- Statusbar
					StatusbarTexture = "HalD",
					StatusbarColor = {1, 0.3, 0.1, 1},
					-- Text
					TimeFont = "accid",
					TextFont = "accid",
					TimeFontSize = 14,
					TextFontSize = 14,
					TimeFontFlags = "NONE",
					TextFontFlags = "NONE",
					TimeFontColor = {GameFontNormal:GetTextColor()},
					TextFontColor = {GameFontNormal:GetTextColor()},
					-- Other
					JumpToSecondaryTime = -1,
					Secondary = "NONE",
					Spacing = 1,
					Height = 35,
					Width = 200,
					GrowUp = true,
					ReverseSorting = false,
					ShowIcon = true,
					IconSide = "LEFT",
					TimeTextSide = "RIGHT",
					Scale = 1,
					JumpSpeed = 1.5,
					FillUp = true,
					IconSpace = 2,
					Animate = true,
					-- Flash
					FlashTime = 10,
					FlashSpeed = 2,
					Flash = true,
					FlashColor = {1, 0.61, 0.35, 1},
				},
				["Default"] = {
				},
			},			
		},
	}
	
	self.db = RW.db:RegisterNamespace(self:GetName(), defaults)
	
	self:Reset()
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")	
end

function plugin:PluginEnable()
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")

	self:RegisterMessage("RW_ConfigMode")
	self:RegisterMessage("RW_StartBar")
	self:RegisterMessage("RW_CancelBar")
	
	self:Reset()
end

function plugin:PluginDisable()
end

function plugin:Reset(event)
	self = plugin
	
	if event and event == "OnProfileReset" then
		for i, d in pairs(db.Anchors) do
			if i ~= "**"  then
				db.Anchors[i] = nil
			end
		end
		local t = db.Anchors.Default
	end
	
	for anchorName, anchor in pairs(Anchors) do		
		anchor:Delete()
		Anchors[anchorName] = nil
	end
	
	db = self.db.profile	
	-- Cleanup from old version
	if db.Master or db.fadeTime then
		self.db.Anchors.Long = nil
		self.db.Anchors.Short = nil
		self.db.Anchors.Normal = nil
		self.db:ResetProfile()		
	end	
	
	for anchorName, d in pairs(db.Anchors) do
		if anchorName ~= "**" then
			local a = self:CreateAnchor(anchorName)
			a:ApplySettings()
		end
	end
	
	DefaultAnchor = Anchors["Default"]
end

function  plugin:GetAnchorList()
	wipe(AnchorList)
	for anchorName, data in pairs(Anchors) do
		if anchorName ~= "**" then
			AnchorList[anchorName] = {text = data.db.Name, value = anchorName}
		end
	end
	return AnchorList
end

function plugin:RW_StartBar(message, id, duration, text, icon, color, anchorID)
	if not id then return end
	if not duration or duration < 0 then return end
	if ActiveBars[id] then
		ActiveBars[id]:Recycle()
	end	
	
	local b = self:CreateBar()
	
	b.id = id	
	
	b.orgtxt = text
	b.nametxt:SetText(Utils:ColorNames(text))
	b:SetIcon(icon)
	b.color = color
	
	b:Start(duration, anchorID)
end

--- Cancel a running bar
-- @param id The id of the bar, same as the one supplied when starting a bar
function plugin:RW_CancelBar(message, id)
	if not id then return end
	local bar = ActiveBars[id]
	if bar then bar:Stop() end
end

do

	function plugin:RW_ConfigMode(message, start)
		for anchorName, anchor in pairs(Anchors) do
			InConfigMode = true
			if start then
				anchor:Show()
				
				for i = 1, 5 do
					local b = self:CreateBar()	
					local spellName, rank, icon = RW.Utils:GetRandomSpell()
					b.id = anchorName..i					
					b.orgtxt = spellName
					b.nametxt:SetText(Utils:ColorNames(spellName))
					b:SetIcon(icon)
					b.color = color				
					b:Start(350, anchorName)
					b:Paus(true)
					b.TimeLeft = (i-1) * 30 + 10
				end
			else
				InConfigMode = nil
				anchor:Hide()
				for i = 1, 5 do
					self:SendMessage("RW_CancelBar", anchorName..i)
				end
			end
		end
	end

	function plugin:GetOptions()		
		return LB.PLUGINS, LB.ALERT_PLUGINS, L.NAME, self.AnchorOptions, [[Interface\Icons\INV_Sigil_Thorim]]
	end	

	-- Tables
	local AllAnchorsList = {}
	local CopyAnchorSettingsList = {}
	local JumpAnchorList = {}
	-- List tables for dropdowns
	local SideList = {
		["LEFT"] = "Left",
		["RIGHT"] = "Right",
	}
	-- Function
	local AnchorSelected
	local TabSelected
	-- Others
	local LastSelectedAnchor
	local LastSelectedTab
	local TheAnchor
	
	function plugin:AnchorOptions()
		local self = plugin		
		
		local scroll = AGU:Create("ScrollFrame")	
		scroll:SetLayout("Flow")
		
		-- Information
		local group = UIF:InlineGroup1()
		local label = UIF:Text1(L.DESC)
		label:SetImage([[Interface\Icons\INV_Sigil_Thorim]])
		group:AddChild(label)
		scroll:AddChild(group)
		
		-- Main anchor select
		wipe(AllAnchorsList)
		for anchorName, data in pairs(Anchors) do
			if anchorName ~= "**" then
				AllAnchorsList[anchorName] = {text = data.db.Name, value = anchorName}
			end
		end
		
		local ed = AGU:Create("EditBox")
		ed:SetLabel(L.CREATE_ANCHOR)
		ed:SetCallback("OnEnterPressed", function(self, event, text)
			local newAnchor = plugin:CreateAnchor(text)
			LastSelectedAnchor = text
			newAnchor:Show()
			self:SetText()
			self.editbox:ClearFocus()
			plugin:SendMessage("RW_GUIValuesChanged")
		end)
		scroll:AddChild(ed)
		
		local button = UIF:Button(L.SHOW_ANCHORS, function()
			for i, d in pairs(Anchors) do
				d:Show()
			end
		end)
		scroll:AddChild(button)
		
		button = UIF:Button(L.HIDE_ANCHORS, function()
			for i, d in pairs(Anchors) do
				d:Hide()
			end
		end)
		scroll:AddChild(button)
		
		local drop = UIF:DropdownGroup("Anchors")
		scroll:AddChild(drop)
		drop:SetLayout("Flow")
		drop:SetGroupList(AllAnchorsList)
		drop:SetCallback("OnGroupSelected", AnchorSelected)
		drop:SetGroup((LastSelectedAnchor and Anchors[LastSelectedAnchor]) and LastSelectedAnchor or "Default")		
			
		return scroll
	end	
	
	AnchorSelected = function(self, event, group)
		self:ReleaseChildren()
		
		LastSelectedAnchor = group
		TheAnchor = Anchors[group]		
		
		local tgroup = UIF:TabGroup1()
		self:AddChild(tgroup)
		tgroup:SetTabs(
		{
			{text = LB.POSITION .." & " .. LB.SIZE, value = "Position"},
			{text = LB.APPEARANCE, value = "Appearance"},
			{text = LB.LAYOUT, value = "Layout"},
			{text = L.FLASH .. " & " .. L.ANIMATION, value = "Flash"},
			{text = "Other", value = "Handle"},
		})
		tgroup:SetCallback("OnGroupSelected", TabSelected)
		tgroup:SelectTab(LastSelectedTab or "Position")				
	end
	
	TabSelected = function(self, event, tab)
		self:ReleaseChildren()
		LastSelectedTab = tab
		
		local function Callback()
			TheAnchor:ApplySettings()
		end
		
		if tab == "Position" then
			local group = UIF:InlineGroup2(LB.SIZE)
			self:AddChild(group)
		
			local slider = UIF:Slider(LB.WIDTH, TheAnchor.db, "Width", 5, 1000, 1, Callback)
			group:AddChild(slider)
			
			slider = UIF:Slider(LB.HEIGHT, TheAnchor.db, "Height", 5, 500, 1, Callback)
			group:AddChild(slider)
			
			local group = UIF:InlineGroup2(LB.POSITION)
			self:AddChild(group)
			
			slider = UIF:Slider("X", TheAnchor.db.Position, 1, 
				-floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 
				floor(GetScreenWidth() * UIParent:GetEffectiveScale()),
				1, Callback)
			group:AddChild(slider)
			
			slider = UIF:Slider("Y", TheAnchor.db.Position, 2, 
				-floor(GetScreenHeight() * UIParent:GetEffectiveScale()), 
				floor(GetScreenHeight() * UIParent:GetEffectiveScale()),
				1, Callback)
			group:AddChild(slider)
			
		elseif tab == "Appearance" then
			local group = UIF:InlineGroup2(L.BARS)
			self:AddChild(group)
			
			local color = UIF:ColorSelect(LB.BACKGROUND_COLOR, TheAnchor.db, "BackgroundColor", Callback, true)
			color:SetRelativeWidth(0.33)
			group:AddChild(color)
			
			color = UIF:ColorSelect(LB.BAR_COLOR, TheAnchor.db, "StatusbarColor", Callback, true)
			color:SetRelativeWidth(0.33)
			group:AddChild(color)
		
			local drop = UIF:LSMDropdown("statusbar", LB.BAR_TEXTURE, TheAnchor.db, "StatusbarTexture", Callback)
			drop:SetRelativeWidth(0.33)
			group:AddChild(drop)		
			
			group = UIF:InlineGroup2(L.BORDER)
			self:AddChild(group)
			
			color = UIF:ColorSelect(LB.BORDER_COLOR, TheAnchor.db, "BorderColor", Callback, true)
			color:SetRelativeWidth(0.33)
			group:AddChild(color)
			
			drop = UIF:LSMDropdown("border", LB.BORDER_TEXTURE, TheAnchor.db, "BorderTexture", Callback)
			drop:SetRelativeWidth(0.33)
			group:AddChild(drop)
			
			local slider = UIF:Slider(LB.WIDTH, TheAnchor.db, "BorderWidth", 0, 50, 1, Callback)
			slider:SetRelativeWidth(0.33)
			group:AddChild(slider)
			
			group = UIF:InlineGroup2(L.TEXT_FONT)
			self:AddChild(group)
			
			drop = UIF:LSMDropdown("font", LB.FONT, TheAnchor.db, "TextFont", Callback)
			drop:SetRelativeWidth(0.5)
			group:AddChild(drop)			
			
			drop = UIF:Dropdown(LB.FONT_OUTLINE, TheAnchor.db, "TextFontFlags", Callback, Utils.Constants.FontFlagList)
			drop:SetRelativeWidth(0.5)
			group:AddChild(drop)
			
			local slider = UIF:Slider(LB.FONT_SIZE, TheAnchor.db, "TextFontSize", 1, 50, 1, Callback)
			slider:SetRelativeWidth(0.5)
			group:AddChild(slider)			
			
			color = UIF:ColorSelect(LB.FONT_COLOR, TheAnchor.db, "TextFontColor", Callback)
			color:SetRelativeWidth(0.5)
			group:AddChild(color)			
			
			group = UIF:InlineGroup2(L.TIMER_TEXT_FONT)
			self:AddChild(group)
			
			drop = UIF:LSMDropdown("font", LB.FONT, TheAnchor.db, "TimeFont", Callback)
			drop:SetRelativeWidth(0.5)
			group:AddChild(drop)			
			
			drop = UIF:Dropdown(LB.FONT_OUTLINE, TheAnchor.db, "TimeFontFlags", Callback, Utils.Constants.FontFlagList)
			drop:SetRelativeWidth(0.5)
			group:AddChild(drop)
			
			local slider = UIF:Slider(LB.FONT_SIZE, TheAnchor.db, "TimeFontSize", 1, 50, 1, Callback)
			slider:SetRelativeWidth(0.5)
			group:AddChild(slider)			
			
			color = UIF:ColorSelect(LB.FONT_COLOR, TheAnchor.db, "TimeFontColor", Callback)
			color:SetRelativeWidth(0.5)
			group:AddChild(color)	
			
		elseif tab == "Layout" then
			local check = UIF:CheckBox(LB.GROW_UP, TheAnchor.db, "GrowUp", Callback)
			check:SetDescription(LB.GROW_UP_DESC)
			self:AddChild(check)
			
			check = UIF:CheckBox(L.FILL_BARS, TheAnchor.db, "FillUp", Callback)
			check:SetDescription(L.FILL_BARS_DESC)
			self:AddChild(check)
			
			check = UIF:CheckBox(L.REVERSE_SORTING, TheAnchor.db, "ReverseSorting", Callback)
			check:SetDescription(L.REVERSE_SORTING_DESC)
			self:AddChild(check)
			
			local slider = UIF:Slider(L.VERTICAL_SPACING, TheAnchor.db, "Spacing", 0, 40, 1, Callback)
			self:AddChild(slider)
			
			slider = UIF:Slider(L.INTERNAL_SPACING, TheAnchor.db, "Insets", 0, 40, 1, Callback)
			self:AddChild(slider)
			
			self:AddChild(UIF:NewLine())
			
			check = UIF:CheckBox(L.SHOW_ICON, TheAnchor.db, "ShowIcon", Callback)
			check:SetRelativeWidth(0.5)
			check:SetDescription(L.SHOW_ICON_DESC)
			self:AddChild(check)
			
			local drop = UIF:Dropdown(L.ICON_POSITION, TheAnchor.db, "IconSide", Callback, SideList)
			drop:SetRelativeWidth(0.5)
			self:AddChild(drop)
			
			drop = UIF:Dropdown(L.TIMER_SIDE, TheAnchor.db, "TimeTextSide", Callback, SideList)
			self:AddChild(drop)
					
		elseif tab == "Flash" then
			wipe(JumpAnchorList)
			for anchorName, data in pairs(Anchors) do
				if anchorName ~= "**" and anchorName ~= LastSelectedAnchor then
					if data.db.Secondary ~= LastSelectedAnchor then
						JumpAnchorList[anchorName] = {text = data.db.Name, value = anchorName}
					end
				end
			end
			JumpAnchorList["NONE"] = {text = "None", value = "NONE"}
		
			local group = UIF:InlineGroup2(L.FLASH)
			self:AddChild(group)
			
			local check = UIF:CheckBox(L.FLASH, TheAnchor.db, "Flash")
			check:SetDescription(L.FLASH_DESC)
			group:AddChild(check)
			
			local slider = UIF:Slider(L.FLASH_STARTTIME, TheAnchor.db, "FlashTime", 0, 100, 1)
			group:AddChild(slider)
			
			slider = UIF:Slider(L.FLASH_SPEED, TheAnchor.db, "FlashSpeed", 0.1, 5, 0.1)
			group:AddChild(slider)
			
			local color = UIF:ColorSelect(L.FLASH_COLOR, TheAnchor.db, "FlashColor", nil, true)
			group:AddChild(color)
			
			group = UIF:InlineGroup2(L.ANIMATION)
			self:AddChild(group)
			
			check = UIF:CheckBox(L.ANIMATE, TheAnchor.db, "Animate")
			check:SetDescription(L.ANIMATE_TRANSITIONS_DESC)
			group:AddChild(check)
			
			local drop = UIF:Dropdown(L.SECONDARY_ANCHOR, TheAnchor.db, "Secondary", Callback, JumpAnchorList)
			group:AddChild(drop)
			
			slider = UIF:Slider(L.SECONDARY_JUMP_TIME, TheAnchor.db, "JumpToSecondaryTime", 0, 100, 1)
			group:AddChild(slider)
			
					
		elseif tab == "Handle" then
			self:SetLayout("List")
			local copyDisabled = true
			wipe(CopyAnchorSettingsList)
			for anchorName, data in pairs(Anchors) do
				if anchorName ~= "**" and anchorName ~= LastSelectedAnchor then
					CopyAnchorSettingsList[anchorName] = {text = data.db.Name, value = anchorName}
					copyDisabled = false
				end
			end
			
			local button = UIF:Button(LB.DELETE, function(self)
				db.Anchors[LastSelectedAnchor] = nil
				Anchors[LastSelectedAnchor] = nil
				TheAnchor:Delete()
			end)
			button:SetUserData("confirm", true)
			button:SetUserData("TriggerGlobalUpdate", true)
			button:SetDisabled(LastSelectedAnchor == "Default")
			self:AddChild(button)
			
			local ed = AGU:Create("EditBox")
			ed:SetLabel(LB.RENAME)
			ed:SetDisabled(LastSelectedAnchor == "Default")
			ed:SetCallback("OnEnterPressed", function(self, event, text)
				TheAnchor:SetName(text)
				self:SetText()
				self.editbox:ClearFocus()
				plugin:SendMessage("RW_GUIValuesChanged")
			end)
			self:AddChild(ed)
			
			local drop = UIF:SimpleDropdown(L.COPY_SETTINGS, function(self, event, value)
				local oldX = TheAnchor.db.Position[1]
				local oldY = TheAnchor.db.Position[2]
				local oldName = TheAnchor.db.Name
				Utils:CopyTableShallow(Anchors[value].db, TheAnchor.db)
				TheAnchor.db.Position[1] = oldX
				TheAnchor.db.Position[2] = oldY
				TheAnchor.db.Name = oldName
				TheAnchor:ApplySettings()
				plugin:SendMessage("RW_GUIValuesChanged")
			end, CopyAnchorSettingsList)
			drop:SetDisabled(copyDisabled)
			drop:SetUserData("confirm", true)
			drop:SetUserData("TriggerGlobalUpdate", true)
			self:AddChild(drop)
		end
		
		self.parent:DoLayout()		
		self:DoLayout()
		self.parent.parent:DoLayout()
	end
end

----------------------------------------------------
-- Bar functions
----------------------------------------------------

do
	local function AlphaStep1Finished(self)		
		self.obj:ApplySettings()
	end
	
	local function AlphaFadeOutFinished(self)
		self.obj.frame:Hide()		
		self.obj.anchor:RemoveBar(self.obj)
		self.obj.anchor = nil
	end
	
	local function BarClicked(self, arg1, arg2)
		if arg1 == "LeftButton" then
			local party = RW:GetPlugin("Party")
			local pType = ((party:GetPartyType() == "raid" and "RAID") or (party:GetPartyType() == "party" and "PARTY") or (party:GetPartyType() == "battleground" and "BATTLEGROUND")) or "SAY"
			if pType then
				SendChatMessage("[RW]: "..self.obj.orgtxt.." - " ..self.obj.timetxt:GetText(), pType)
			end
		else
			if self.obj.id then
				self.obj:Stop()
			end
		end
	end
	
	local function BarEvent(self, event, arg1, arg2)
		if arg1 == "LSHIFT" or arg1 == "RSHIFT" then
			if arg2 == 1 then
				self:EnableMouse(true)
			else
				self:EnableMouse(false)
			end
		end
	end

	function plugin:CreateBar()
		local newBar = tremove(Cache)
		
		if not newBar then	
			
			local frame = CreateFrame("frame", nil, UIParent)	
			frame:SetScript("OnEvent", BarEvent)
			frame:RegisterEvent("MODIFIER_STATE_CHANGED")
			frame:SetScript("OnMouseDown", BarClicked)
			
			local bg = frame:CreateTexture(nil, "BACKGROUND")
			
			local bar = CreateFrame("StatusBar", nil, frame)
			bar:SetMinMaxValues(0, 1)
			
			local icon = frame:CreateTexture(nil, "ARTWORK")
			icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
			
			local timeText = bar:CreateFontString(nil, "Artwork", "GameFontNormal")
			
			local nameText = bar:CreateFontString(nil, nil, "GameFontNormal")
			nameText:SetJustifyH("LEFT")
			
			newBar = setmetatable({
				frame = frame,
				bar = bar,
				bg = bg,
				icon = icon,				
				timetxt = timeText,
				nametxt = nameText,
			}, {__index = barProt})	
				
			tinsert(AllBars, newBar)	

			frame.obj = newBar
			
			local ag, translation, alpha
				
			
			-- Switch Alpha
			ag = frame:CreateAnimationGroup()
			newBar.frame.SwitchAlpha = ag
			
			alpha = ag:CreateAnimation("Alpha")
			alpha:SetChange(-1)
			alpha:SetDuration(0.5)
			alpha:SetSmoothing("IN_OUT")
			alpha:SetOrder(1)
			alpha.obj = newBar
			alpha:SetScript("OnFinished", AlphaStep1Finished)
			newBar.frame.AlphaStep1 = alpha
			
			alpha = ag:CreateAnimation("Alpha")
			alpha:SetChange(1)
			alpha:SetDuration(0.5)
			alpha:SetSmoothing("IN_OUT")
			alpha:SetOrder(2)
			alpha.obj = newBar
			alpha:SetScript("OnFinished", AlphaStep2Finished)			
			newBar.frame.AlphaStep2 = alpha
			
			-- Fade in alpha
			ag = frame:CreateAnimationGroup()
			newBar.frame.FadeInAlphaAG = ag
			
			alpha = ag:CreateAnimation("Alpha")
			alpha:SetChange(-1)
			alpha:SetDuration(0)
			alpha:SetOrder(1)
			alpha.obj = newBar
			alpha:SetScript("OnFinished", AlphaStep2Finished)			
			newBar.frame.FadeInAlphaStep1 = alpha
			
			alpha = ag:CreateAnimation("Alpha")
			alpha:SetChange(1)
			alpha:SetDuration(0.5)
			alpha:SetSmoothing("IN_OUT")
			alpha:SetOrder(2)
			alpha.obj = newBar		
			newBar.frame.FadeInAlphaStep2 = alpha
			
			-- Fade out alpha
			ag = frame:CreateAnimationGroup()
			newBar.frame.FadeOutAlphaAG = ag
			
			alpha = ag:CreateAnimation("Alpha")
			alpha:SetChange(-1)
			alpha:SetDuration(1)
			alpha:SetSmoothing("IN_OUT")
			alpha.obj = newBar
			alpha:SetScript("OnFinished", AlphaFadeOutFinished)			
			newBar.frame.FadeOutAlpha = alpha
		end
		
		return newBar
	end
end

function barProt:Start(duration, anchorID)
	self.TotalTime = duration
	self.TimeLeft = duration
	self.Started = time()
	self.fresh = true
	
	if anchorID and Anchors[anchorID] then
		Anchors[anchorID]:AddBar(self)		
	else
		DefaultAnchor:AddBar(self)		
	end	
	
	self:ApplySettings()	
	self.frame.FadeOutAlphaAG:Stop()
	self.frame.FadeInAlphaAG:Play()
	self.frame:Show()
	
	ActiveBars[self.id] = self
end

function barProt:Stop()
	self:Recycle()
end

function barProt:Paus(enable)
	self.paused = enable
end

function barProt:SetRemainingTime(remaining)
	self.TimeLeft = remaining
end

function barProt:SetTotalTime(totalTime)
	self.TotalTime = totalTime
end

-- Set the progress of the bar. If value is passed it will be used.
-- else the value will be calculated from the TimeLeft and TotalTime variables
function barProt:SetValue(value)
	if value then
		self.bar:SetValue(value)
	else
		if self.anchor.db.FillUp then
			self.bar:SetValue(1 - self.TimeLeft / self.TotalTime)
		else
			self.bar:SetValue(self.TimeLeft / self.TotalTime)
		end	
	end
end

function barProt:SetIcon(icon)
	self.icon:SetTexture(icon)
end

function barProt:Update(elapsed)	
	if not self.TimeLeft then return end
	local aDB = self.anchor and self.anchor.db or db.Anchors["**"]
	
	-- Count the remaining time down
	if not self.paused then
		self.TimeLeft = self.TimeLeft - elapsed
	end
	
	self.timetxt:SetText(Utils:TimeToStringShort(self.TimeLeft))
	
	
	if self.animate then
		if self.animate > 0 then
			if time() - self.Started < 1 then
				--self:SetPosition()
				--self.frame.FadeOutAlphaAG:Stop()
				--self.frame.FadeInAlphaAG:Stop()
				--self.frame.SwitchAlpha:Finish()
				self.animate = self.animate - elapsed
				self:Animate(elapsed)
			else
				self.animate = self.animate - elapsed
				self:Animate(elapsed)
			end
		else
			self:SetPosition()
			self.animate = nil
		end
	end
	
	-- IF it has ended, then recycle it.
	if self.TimeLeft < 0 then		
		self.timetxt:SetText(0)
		self:Stop()		
	else	
		self:SetValue()
		
		-- Anchor Switch
		if self.TimeLeft <= aDB.JumpToSecondaryTime and aDB.Secondary and Anchors[aDB.Secondary] then
			self.TotalTime = aDB.JumpToSecondaryTime
			self:SetValue(0)
			Anchors[aDB.Secondary]:AddBar(self)
			if self.anchor.db.Animate then		
				self.frame.FadeInAlphaAG:Stop()
				self.frame.SwitchAlpha:Play()
			else
				self:ApplySettings()
			end			
		end
		
		-- Color Flash
		if self.TimeLeft < aDB.FlashTime and aDB.Flash then
			self.FlashTime = self.FlashTime or 0
			local perc
			if self.FlashTime <= aDB.FlashSpeed / 2 then
				perc = self.FlashTime / (aDB.FlashSpeed / 2)
			else
				perc = (aDB.FlashSpeed - self.FlashTime) / (aDB.FlashSpeed / 2)
			end
					
			if self.FlashDir then
				if self.FlashTime > aDB.FlashSpeed then
					self.FlashDir = false
				end
			else
				if self.FlashTime < 0 then
					self.FlashDir = true
				end
			end	
			
			local r, g, b
			if self.color then
				r, g, b = unpack(self.color)
			else
				r, g, b = unpack(aDB.StatusbarColor)
			end
			
			self.bar:SetStatusBarColor(Utils:ColorGradientSimple(perc, r, g, b, unpack(aDB.FlashColor)))
			
			if self.FlashDir then
				self.FlashTime = self.FlashTime + elapsed
			else
				self.FlashTime = self.FlashTime - elapsed
			end
		end
	end
end

function barProt:Recycle()	
	ActiveBars[self.id] = nil
	self:Reset()

	self.frame.FadeOutAlphaAG:Play()
	
	tinsert(Cache, self)
end

function barProt:Reset()		
	self.TotalTime = nil
	self.TimeLeft = nil
	self.id = nil
	self.orgtxt = nil
	self.color = nil
	self.FlashDir = nil
	self.order = nil
	self.animate = nil
	self.FlashTime = nil
	self.paused = nil
	self.Started = nil
end

function barProt:ApplySettings()	
	local aDB = self.anchor and self.anchor.db or db.Anchors["**"]
	
	-- Clear positions
	self.timetxt:ClearAllPoints()
	self.nametxt:ClearAllPoints()
	self.bar:ClearAllPoints()
	self.icon:ClearAllPoints()
	self.bg:ClearAllPoints()
	
	-- Set textures
	self.bg:SetTexture(unpack(aDB.BackgroundColor))
	self.bar:SetStatusBarTexture(LSM:Fetch("statusbar", aDB.StatusbarTexture))
	
	-- Set Backdrop
	backdrop.edgeFile = LSM:Fetch("border", aDB.BorderTexture)
	backdrop.edgeSize = aDB.BorderWidth
	self.frame:SetBackdrop(backdrop)
	
	-- Set Colors
	self.bar:GetStatusBarTexture():SetHorizTile(false)
	if self.color then
		self.bar:SetStatusBarColor(unpack(self.color))
	else
		self.bar:SetStatusBarColor(unpack(aDB.StatusbarColor))
	end
	self.timetxt:SetTextColor(unpack(aDB.TimeFontColor))
	self.nametxt:SetTextColor(unpack(aDB.TextFontColor))
	self.frame:SetBackdropBorderColor(unpack(aDB.BorderColor))
	
	-- Set Fonts
	self.timetxt:SetFont(LSM:Fetch("font", aDB.TimeFont), aDB.TimeFontSize, aDB.TimeFontFlags)
	self.nametxt:SetFont(LSM:Fetch("font", aDB.TextFont), aDB.TextFontSize, aDB.TextFontFlags)
	
	-- Set Points
	self.bg:SetPoint("TOPLEFT", aDB.Insets, -aDB.Insets)
	self.bg:SetPoint("BOTTOMRIGHT", -aDB.Insets, aDB.Insets)
	
	if aDB.ShowIcon and self.icon:GetTexture() then
		self.icon:Show()
		
		if aDB.IconSide == "LEFT" then		
			self.icon:SetPoint("TOPLEFT", self.bg)			
			
			self.bar:SetPoint("TOPLEFT", self.icon, "TOPRIGHT", aDB.IconSpace, 0)
			self.bar:SetPoint("BOTTOMRIGHT", self.bg)
		else
			self.icon:SetPoint("TOPRIGHT", self.bg)
			
			self.bar:SetPoint("TOPLEFT", self.bg)
			self.bar:SetPoint("BOTTOMRIGHT", self.icon, "BOTTOMLEFT", -aDB.IconSpace, 0)
		end
		
		self.icon:SetPoint("BOTTOM", self.bg)
		self.icon:SetWidth(aDB.Height - aDB.Insets * 2)
	else
		self.icon:Hide()
		
		self.bar:SetPoint("TOPLEFT", self.bg)
		self.bar:SetPoint("BOTTOMRIGHT", self.bg)
	end
	
	if aDB.TimeTextSide == "LEFT" then
		self.timetxt:SetPoint("LEFT", 5, 0)
		self.timetxt:SetJustifyH("LEFT")
		
		self.nametxt:SetPoint("LEFT", self.timetxt, "RIGHT", 5, 0)
		self.nametxt:SetPoint("RIGHT", -5, 0)
		self.nametxt:SetJustifyH("RIGHT")
	else
		self.timetxt:SetPoint("RIGHT", self.bar, -5, 0)
		--self.timetxt:SetPoint("BOTTOMRIGHT", self.bar, -5, 0)
		self.timetxt:SetJustifyH("RIGHT")
		
		self.nametxt:SetPoint("LEFT", 5, 0)
		self.nametxt:SetPoint("RIGHT", self.timetxt, "LEFT", -5, 0)
		self.nametxt:SetJustifyH("LEFT")
	end
	
	
	-- Set Size
	self.frame:SetWidth(aDB.Width)
	self.frame:SetHeight(aDB.Height)
	
end

-- Animate the bar when moving
function barProt:Animate(elapsed)
	local aDB = self.anchor and self.anchor.db or db.Anchors["**"]
	local anchor = self.anchor.anchor
	
	-- Get Start and Target coords (Y value needs more calculation since it depends on
	-- if bars grow up or down, spacing and order.
	local startX, startY = self.frame:GetLeft(), self.frame:GetBottom()
	local targetX, targetY = anchor:GetLeft(), 0
	
	local order = self.order
	if aDB.GrowUp then
		targetY = anchor:GetTop() + aDB.Height * (order - 1) + aDB.Spacing * (order - 1)
	else
		targetY = anchor:GetTop() - aDB.Height * order - aDB.Spacing * (order - 1) - anchor:GetHeight()
	end
	
	-- Linearpolate the movement 
	local newX = Utils:Lerp(startX, targetX, elapsed * aDB.JumpSpeed)
	local newY = Utils:Lerp(startY, targetY, elapsed * aDB.JumpSpeed)
	
	local diffx = -(targetX - newX)
	local diffy = -(targetY - newY)
	
	if aDB.GrowUp then
		self.frame:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", diffx, diffy + aDB.Height * (order - 1) + aDB.Spacing * (order - 1))
	else
		self.frame:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", diffx, diffy - aDB.Height * order - aDB.Spacing * (order - 1) - anchor:GetHeight())
	end
end

function barProt:SetPosition()
	local aDB = self.anchor and self.anchor.db or db.Anchors["**"]
	if not aDB.Height then return end
	local i = self.order
	if aDB.GrowUp then
		self.frame:SetPoint("BOTTOMLEFT", self.anchor.anchor, "TOPLEFT", 0, aDB.Spacing * (i - 1) + aDB.Height * (i - 1))
	else
		self.frame:SetPoint("BOTTOMLEFT", self.anchor.anchor, "TOPLEFT", 0, -self.anchor.anchor:GetHeight() - aDB.Height * i - aDB.Spacing * (i - 1))
	end
end


----------------------------------------------------
-- Anchor functions
----------------------------------------------------

local function barSort(a, b)
	if a.TimeLeft == b.TimeLeft  or not a.TimeLeft or not b.TimeLeft then
		return a.nametxt:GetText() < b.nametxt:GetText()
	end
	return a.TimeLeft > b.TimeLeft
end

local function barSortReverse(a, b)
	if a.TimeLeft == b.TimeLeft  or not a.TimeLeft or not b.TimeLeft then
		return a.nametxt:GetText() < b.nametxt:GetText()
	end
	return a.TimeLeft < b.TimeLeft
end

function plugin:CreateAnchor(name)
	if not Anchors[name] then
		local f = Utils:CreateAnchor(name, UIParent, "Bar: "..name)
		f.name = name
		f:SetScript("OnDragStart", dragStart)
		f:SetScript("OnDragStop", dragStop)
				
		local newAnchor = setmetatable({
			anchor = f,
			bars = {},
		}, {__index = anchorProt})
		Anchors[name] = newAnchor
		
		local db = db.Anchors[name]		
		db.Name = db.Name or name
		
		newAnchor.db = db
		newAnchor:SetName(newAnchor.db.Name)
		
		f:SetWidth(db.Width)
	end	
	
	if InConfigMode then
		self:RW_ConfigMode(nil, false)
		self:RW_ConfigMode(nil, true)
	end
	
	return Anchors[name]
end

function anchorProt:AddBar(bar)
	if bar.anchor then
		bar.anchor:RemoveBar(bar)
	end
	bar.order = nil
	bar.animate = nil
	bar.anchor = self
	tinsert(self.bars, bar)
	self:SortBars()
end

function anchorProt:RemoveBar(bar)
	Utils:TRemove(self.bars, bar)
	self:SortBars()
end

function anchorProt:Delete()
	for i, bar in pairs(self.bars) do		
		bar:Stop()
	end	
	self:Hide()
end

function anchorProt:SortBars()
	local aDB = self.db	
	
	if aDB.ReverseSorting then
		table.sort(self.bars, barSort)
	else
		table.sort(self.bars, barSortReverse)
	end
	
	for i, bar in pairs(self.bars) do
				
		if bar.fresh then
			bar.order = i
			bar:SetPosition()
			bar.fresh = nil		
		elseif (not bar.order or bar.order ~= i or bar.animate) and aDB.Animate then
			bar.order = i
			bar.animate = 4
		else
			bar.order = i
			bar:SetPosition()
		end
	end
end

function anchorProt:Hide()
	self.anchor:Hide()
end

function anchorProt:Show()
	self.anchor:Show()
end

function anchorProt:SetName(name)
	self.anchor.name = name
	self.db.Name = name
	self:SetText("Bar: "..name)
end

function anchorProt:SetText(value)
	self.anchor.str:SetText(value)
end

function anchorProt:ApplySettings()
	local aDB = self.db

	for i, bar in pairs(self.bars) do
		bar:ApplySettings()
		bar.order = nil
	end
	
	self.anchor:ClearAllPoints()		
	self.anchor:SetPoint("CENTER", aDB.Position[1], aDB.Position[2])
	self.anchor:SetWidth(aDB.Width)
	
	self:SortBars()	
end

----------------------------------------------------
-- Update
----------------------------------------------------
dragStart = function(self)
	self:StartMoving()
end

dragStop = function(self)
	self:StopMovingOrSizing()
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = self:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	x = x * scale
	y = y * scale
	
	local aDb = db.Anchors[self.name]
	
	aDb.Position[1], aDb.Position[2] = x, y
	
	plugin:SendMessage("RW_GUIValuesChanged")
end

UpdateFrame:SetScript("OnUpdate", function(self, elapsed)
	for id, bar in pairs(ActiveBars) do
		bar:Update(elapsed)
	end
end)