local mod = RW:Boss("Halion the Twilight Destroyer", "The Ruby Sanctum")
local L

mod:SetPackIcon([[Interface\ICONS\spell_nature_elementalprecision_2]])
mod.BossIcon = [[Interface\Icons\spell_shadow_twilight]]

---------------------------
-- Variables

-- Fiery Combustion
local wFireCon, mFireCon, tFireConCD, iFireCon

-- Soul Consumption
local wShadowCon, mShadowCon, tShadowConCD, iShadowCon

-- Twilight Cutter
local wCutter, tCutterCD, tCutterDur

-- Meteor Strike
local wMeteor, tMeteorCD

-- Dark Breath
local wDarkBreath, tDarkBreathCD

-- Flame Breath
local wFlameBreath, tFlameBreathCD

local mPhase

---------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L.name = "Halion the Twilight Destroyer"
		
		L.twilightcutter_emote = "The orbiting spheres pulse with dark energy!"
		
		L.phase2_emote = "You will find only suffering within the realm of twilight! Enter if you dare!"
	end
	L = mod:GetLocale()
	self.L = L
	
	self:SetTrigger(39863, 40141)
	self:SetWin()
	
	-- Fiery Combustion
	tFireConCD			= mod:CDTimer(20, 74562)
	wFireCon			= mod:OnYouWarning(74562)
	mFireCon			= mod:TargetMessage(74562)
	iFireCon			= mod:Icon("SKULL", 74562, false, 30)
	
	-- Soul Consumption
	tShadowConCD		= mod:CDTimer(20, 74792)
	wShadowCon			= mod:OnYouWarning(74792)
	mShadowCon			= mod:TargetMessage(74792)
	iShadowCon			= mod:Icon("CROSS", 74792, false, 30)
	
	-- Twilight Cutter
	wCutter				= mod:CastWarning(77846)
	tCutterCD			= mod:CDTimer(20, 77846)
	tCutterDur			= mod:DurTimer(10, 77846)
	
	-- Meteor Strike
	tMeteorCD			= mod:CDTimer(35, 74648)
	wMeteor				= mod:CastWarning(74648)
	
	-- Dark Breath
	wDarkBreath			= mod:CastWarning(75954, nil, nil, false)
	tDarkBreathCD		= mod:CDTimer(12, 75954, nil, nil, false)
	
	-- Flame Breath
	wFlameBreath		= mod:CastWarning(74526, nil, nil, false)
	tFlameBreathCD		= mod:CDTimer(12, 74526, nil, nil, false)
	
	mPhase				= mod:PhaseMessage()
end

---------------------------
-- Locals

---------------------------
-- Start

function mod:OnEngage(trigger)
	self:Track("FieryConsumption", 74562)
	self:Track("SoulConsumtpion", 74792)
	self:Track("MeteorStrike", 74648, 74637, 75877, 75878, 75879)
	self:Track("DarkBreath", 75954, 74806, 75955, 75956)
	self:Track("FlameBreath", 74526, 74525, 74527, 74528)
	
	self:YellTrack("TwilightCutter", L.twilightcutter_emote)
	self:YellTrack("Phase2", L.phase2_emote)
	
	tMeteorCD:Start(30)
	tFireConCD:Start(17)
	tFlameBreathCD:Start(5.5)
end

function mod:FieryConsumption(event, args)
	if event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			wFireCon:Show()
		end
		mFireCon:Show(args.dName)
		tFireConCD:Start()
		iFireCon:Show(args.dName)
	
	elseif event == "SPELL_AURA_REMOVED" then
		iFireCon:Hide(args.dName)
	end
end

function mod:SoulConsumtpion(event, args)
	if event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			wShadowCon:Show()
		end
		mShadowCon:Show(args.dName)
		tShadowConCD:Start()
		iShadowCon:Show(args.dName)
	
	elseif event == "SPELL_AURA_REMOVED" then
		iShadowCon:Hide(args.dName)
	end
end

function mod:MeteorStrike(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		tMeteorCD:Start()
		wMeteor:Show()
	end
end

function mod:DarkBreath(event, args)
	if event == "SPELL_CAST_START" then
		wDarkBreath:Show()
		tDarkBreathCD:Start()
	end
end

function mod:FlameBreath(event, args)
	if event == "SPELL_CAST_START" then
		wFlameBreath:Show()
		tFlameBreathCD:Start()
	end
end

function mod:TwilightCutter(event, msg, name)
	wCutter:Show()
	tCutterDur:Start()
	tCutterCD:Schedule(10)
end

function mod:Phase2(event, msg, name)
	mPhase:Show(2)
end
